@@ -21,6 +21,8 @@ along with PStreams; if not, write to the Free Software Foundation, Inc.,
 
 
 // TODO test rpstream more
+// TODO test whether error_ cleared after successful open().
+// TODO more tests for vector open()
 
 // test for failures. test opening pstream with neither pstdin nor pstdout.
 // maybe set failbit if !(mode&(pstdin|pstdout|pstderr)) ?
@@ -251,7 +253,13 @@ int main()
 
     {
         // test reading from bidirectional pstream
+
+#if defined(__sun)
+        // Solaris' grep doesn't like "--" and "-"
+        const string cmd = "grep '^127' /etc/hosts /no/such/file /dev/stdin";
+#else
         const string cmd = "grep '^127' -- /etc/hosts /no/such/file -";
+#endif
         pstream ps(cmd, all3streams);
 
         print_result(ps, ps.is_open());
@@ -274,7 +282,13 @@ int main()
     {
         // test input on bidirectional pstream
         // and test child moves onto next file after peof on stdin
+
+#if defined (__sun)
+        // Solaris' grep doesn't like "--" and "-"
+        const string cmd = "grep fnord /etc/hosts /dev/stdin";
+#else
         const string cmd = "grep fnord -- /etc/hosts -";
+#endif
         pstream ps(cmd, all3streams);
 
         print_result(ps, ps.is_open());
@@ -374,6 +388,27 @@ int main()
     //string badcmd = "hgfhdgf";
     const string badcmd = "hgfhdgf 2>/dev/null";
 
+    {
+        // check is_open() works 
+        ipstream is(badcmd);
+        // print_result(is, !is.is_open());  // XXX cannot pass this test!
+#if defined (__sun) || defined(__APPLE__)
+        // fail next test if OS slow to terminate child process, need sleep(1)
+        sleep(1);
+#endif
+        print_result(is, is.rdbuf()->exited() && !is.is_open());
+    }
+
+    {
+        // check is_open() works 
+        pstreams::argv_type argv;
+        argv.push_back("hdhdhd");
+        argv.push_back("arg1");
+        argv.push_back("arg2");
+        ipstream ifail("hdhdhd", argv);
+        print_result(ifail, !ifail.is_open());
+    }
+
     {
         // check eof() works 
         ipstream is(badcmd);
@@ -384,7 +419,9 @@ int main()
     {
         // test writing to bad command
         opstream ofail(badcmd);
+#if defined (__sun) || defined(__APPLE__)
         sleep(1);  // give shell time to try command and exit
+#endif
         // this would cause SIGPIPE: ofail<<"blahblah";
         // does not show failure: print_result(ofail, !ofail.is_open());
         pstreambuf* buf = ofail.rdbuf();
@@ -403,7 +440,7 @@ int main()
         ipstream ifail("hdhdhd", argv);
         check_fail(ifail>>str);
     }
-    
+
     clog << "# Testing behaviour with uninit'ed streams" << endl;
 
     {
@@ -448,6 +485,15 @@ int main()
         print_result(s, cmd == s.command());
     }
 
+
+    {
+        // testing streambuf::in_avail()
+        ipstream in("hostname");
+        streamsize avail = in.rdbuf()->in_avail();
+        cout << "STDOUT: " << avail << " characters: " << in.rdbuf();
+        print_result(in, avail > 0);
+    }
+
     // TODO more testing of other members
 
     clog << "# Testing writing to closed stream\n";
@@ -587,8 +633,12 @@ int main()
 #endif
 
     clog << "# Testing resources freed correctly\n";
+    // TODO repeat tests for vector open()
 
     {
+        const int next_fd = dup(0);
+        ::close(next_fd);
+
         ipstream in("hostname");
 
         ::sleep(3);  // wait for process to exit
@@ -596,21 +646,24 @@ int main()
         in.rdbuf()->exited();  // test for exit, destroy buffers
 
         // check no open files except for stdin, stdout, stderr
-        int next_fd = dup(0);
-        print_result(in, next_fd == 3);
-        ::close(next_fd);
+        int fd = dup(0);
+        print_result(in, next_fd == fd);
+        ::close(fd);
     }
 
     {
+        const int next_fd = dup(0);
+        ::close(next_fd);
+
         pstream p("cat", all3streams);
 
         ::sleep(3);  // wait for process to exit
         p.rdbuf()->exited();  // test for exit, destroy buffers
 
         // check no open files except for stdin, stdout, stderr
-        int next_fd = dup(0);
-        print_result(p, next_fd == 3);
-        ::close(next_fd);
+        int fd = dup(0);
+        print_result(p, next_fd == fd);
+        ::close(fd);
 
         // now close and reopen and check again
 
@@ -620,9 +673,9 @@ int main()
         ::sleep(3);  // wait for process to exit
         p.rdbuf()->exited();  // test for exit, destroy buffers
 
-        next_fd = dup(0);
-        print_result(p, next_fd == 3);
-        ::close(next_fd);
+        fd = dup(0);
+        print_result(p, next_fd == fd);
+        ::close(fd);
     }
 
     return 0;
